@@ -14,18 +14,12 @@ from xpra.os_util import bytestostr, OSX
 from xpra.gtk_common.gtk_util import ensure_item_selected, menuitem, popup_menu_workaround, CheckMenuItemClass, MESSAGE_QUESTION, BUTTONS_NONE
 from xpra.client.client_base import EXIT_OK
 from xpra.gtk_common.about import about, close_about
-from xpra.codecs.loader import PREFERED_ENCODING_ORDER, get_encoding_help, get_encoding_name
+from xpra.codecs.codec_constants import PREFERED_ENCODING_ORDER
+from xpra.codecs.loader import get_encoding_help, get_encoding_name
 from xpra.simple_stats import std_unit_dec
 from xpra.platform.gui import get_icon_size
-try:
-    from xpra.clipboard.translated_clipboard import TranslatedClipboardProtocolHelper
-except ImportError:
-    TranslatedClipboardProtocolHelper = None
-try:
-    from xpra import clipboard
-    HAS_CLIPBOARD = bool(clipboard)
-except ImportError:
-    HAS_CLIPBOARD = False
+from xpra.client import mixin_features
+
 
 from xpra.log import Logger
 log = Logger("menu")
@@ -36,6 +30,7 @@ bandwidthlog = Logger("bandwidth", "network")
 
 HIDE_DISABLED_MENU_ENTRIES = OSX
 
+SHOW_TITLE_ITEM = envbool("XPRA_SHOW_TITLE_ITEM", True)
 SHOW_VERSION_CHECK = envbool("XPRA_SHOW_VERSION_CHECK", True)
 SHOW_UPLOAD = envbool("XPRA_SHOW_UPLOAD_MENU", True)
 STARTSTOP_SOUND_MENU = envbool("XPRA_SHOW_SOUND_MENU", True)
@@ -43,7 +38,7 @@ WEBCAM_MENU = envbool("XPRA_SHOW_WEBCAM_MENU", True)
 RUNCOMMAND_MENU = envbool("XPRA_SHOW_RUNCOMMAND_MENU", True)
 SHOW_SERVER_COMMANDS = envbool("XPRA_SHOW_SERVER_COMMANDS", True)
 SHOW_TRANSFERS = envbool("XPRA_SHOW_TRANSFERS", True)
-SHOW_CLIPBOARD_MENU = envbool("XPRA_SHOW_CLIPBOARD_MENU", HAS_CLIPBOARD)
+SHOW_CLIPBOARD_MENU = envbool("XPRA_SHOW_CLIPBOARD_MENU", True)
 SHOW_SHUTDOWN = envbool("XPRA_SHOW_SHUTDOWN", True)
 WINDOWS_MENU = envbool("XPRA_SHOW_WINDOWS_MENU", True)
 
@@ -277,27 +272,34 @@ class GTKTrayMenuBase(object):
         self.menu_shown = False
         self.menu_icon_size = get_icon_size()
         menu = gtk.Menu()
-        menu.set_title(self.client.session_name or u"Xpra")
-        def set_menu_title(*_args):
-            #set the real name when available:
-            m = self.menu
-            if m:
-                m.set_title(self.client.session_name or u"Xpra")
-        self.client.after_handshake(set_menu_title)
+        title_item = None
+        if SHOW_TITLE_ITEM:
+            title_item = gtk.MenuItem()
+            title_item.set_label(self.client.session_name or u"Xpra")
+            set_sensitive(title_item, False)
+            menu.append(title_item)
+            def set_menu_title(*_args):
+                #set the real name when available:
+                try:
+                    title = self.client.get_tray_title()
+                except:
+                    title = self.client.session_name or u"Xpra"
+                title_item.set_label(title)
+            self.client.after_handshake(set_menu_title)
 
         menu.append(self.make_infomenuitem())
         menu.append(self.make_featuresmenuitem())
-        if self.client.keyboard_helper:
+        if mixin_features.windows and self.client.keyboard_helper:
             menu.append(self.make_layoutsmenuitem())
-        if SHOW_CLIPBOARD_MENU:
+        if mixin_features.clipboard and SHOW_CLIPBOARD_MENU:
             menu.append(self.make_clipboardmenuitem())
-        if self.client.windows_enabled:
+        if mixin_features.windows:
             menu.append(self.make_picturemenuitem())
-        if STARTSTOP_SOUND_MENU:
+        if mixin_features.audio and STARTSTOP_SOUND_MENU:
             menu.append(self.make_audiomenuitem())
-        if WEBCAM_MENU:
+        if mixin_features.webcam and WEBCAM_MENU:
             menu.append(self.make_webcammenuitem())
-        if self.client.windows_enabled and WINDOWS_MENU:
+        if mixin_features.windows and WINDOWS_MENU:
             menu.append(self.make_windowsmenuitem())
         if RUNCOMMAND_MENU or SHOW_SERVER_COMMANDS or SHOW_UPLOAD or SHOW_SHUTDOWN:
             menu.append(self.make_servermenuitem())
@@ -400,27 +402,34 @@ class GTKTrayMenuBase(object):
     def make_bugreportmenuitem(self):
         def show_bug_report_cb(*_args):
             self.show_bug_report()
-        return  self.handshake_menuitem("Bug Report", "bugs.png", None, show_bug_report_cb)
+        return  self.menuitem("Bug Report", "bugs.png", None, show_bug_report_cb)
 
 
     def make_featuresmenuitem(self):
-        features_menu_item = self.menuitem("Features", "features.png")
+        features_menu_item = self.handshake_menuitem("Features", "features.png")
         menu = gtk.Menu()
+        self.append_featuresmenuitems(menu)
         features_menu_item.set_submenu(menu)
         self.popup_menu_workaround(menu)
+        features_menu_item.show_all()
+        return features_menu_item
+
+    def append_featuresmenuitems(self, menu):
         menu.append(self.make_sharingmenuitem())
         menu.append(self.make_lockmenuitem())
-        menu.append(self.make_readonlymenuitem())
-        menu.append(self.make_bellmenuitem())
-        menu.append(self.make_notificationsmenuitem())
-        if self.client.windows_enabled:
+        if mixin_features.windows:
+            menu.append(self.make_readonlymenuitem())
+            menu.append(self.make_bellmenuitem())
+        if mixin_features.notifications:
+            menu.append(self.make_notificationsmenuitem())
+        if mixin_features.windows:
             menu.append(self.make_cursorsmenuitem())
         if self.client.client_supports_opengl:
             menu.append(self.make_openglmenuitem())
-        if self.client.windows_enabled:
+        if mixin_features.windows:
+            menu.append(self.make_modalwindowmenuitem())
+        if mixin_features.windows and self.client.keyboard_helper:
             menu.append(self.make_keyboardsyncmenuitem())
-        features_menu_item.show_all()
-        return features_menu_item
 
     def make_sharingmenuitem(self):
         def sharing_toggled(*args):
@@ -687,23 +696,28 @@ class GTKTrayMenuBase(object):
             kh = self.client.keyboard_helper
             if not kh:
                 self.keyboard_sync_menuitem.set_tooltip_text("Keyboard support is not loaded")
+            elif not self.client.server_toggle_keyboard_sync:
+                self.keyboard_sync_menuitem.set_tooltip_text("Server does not support toggling the synchronization")
             elif kh.keyboard_sync:
                 self.keyboard_sync_menuitem.set_tooltip_text("Disable keyboard synchronization (prevents spurious key repeats on high latency connections)")
             else:
                 self.keyboard_sync_menuitem.set_tooltip_text("Enable keyboard state synchronization")
         def keyboard_sync_toggled(*args):
-            self.client.keyboard_sync = self.keyboard_sync_menuitem.get_active()
-            log("keyboard_sync_toggled(%s) keyboard_sync=%s", args, self.client.keyboard_sync)
-            set_keyboard_sync_tooltip()
-            self.client.emit("keyboard-sync-toggled")
+            ks = self.keyboard_sync_menuitem.get_active()
+            if self.client.keyboard_sync!=ks:
+                self.client.keyboard_sync = ks
+                log("keyboard_sync_toggled(%s) keyboard_sync=%s", args, ks)
+                set_keyboard_sync_tooltip()
+                self.client.send_keyboard_sync_enabled_status()
         self.keyboard_sync_menuitem = self.checkitem("Keyboard Synchronization", keyboard_sync_toggled)
         set_sensitive(self.keyboard_sync_menuitem, False)
         def set_keyboard_sync_menuitem(*args):
             kh = self.client.keyboard_helper
-            set_sensitive(self.keyboard_sync_menuitem, bool(kh))
+            can_set_sync = kh and self.client.server_keyboard and self.client.server_toggle_keyboard_sync
+            set_sensitive(self.keyboard_sync_menuitem, can_set_sync)
             if kh:
                 log("set_keyboard_sync_menuitem%s enabled=%s", args, kh.keyboard_sync)
-            self.keyboard_sync_menuitem.set_active(bool(kh) and bool(kh.keyboard_sync))
+            self.keyboard_sync_menuitem.set_active(kh and bool(kh.keyboard_sync))
             set_keyboard_sync_tooltip()
         self.client.after_handshake(set_keyboard_sync_menuitem)
         return self.keyboard_sync_menuitem
@@ -722,8 +736,22 @@ class GTKTrayMenuBase(object):
         self.client.after_handshake(gl_set)
         return gl
 
+    def make_modalwindowmenuitem(self):
+        modal = self.checkitem("Modal Windows")
+        modal.set_tooltip_text("honour modal windows")
+        modal.set_active(self.client.modal_windows)
+        set_sensitive(modal, False)
+        def modal_toggled(*args):
+            self.client.modal_windows = modal.get_active()
+            log("modal_toggled%s modal_windows=%s", args, self.client.modal_windows)
+        def set_modal_menuitem(*args):
+            set_sensitive(modal, True)
+        self.client.after_handshake(set_modal_menuitem)
+        modal.connect("toggled", modal_toggled)
+        return modal
+
     def make_picturemenuitem(self):
-        picture_menu_item = self.menuitem("Picture", "picture.png")
+        picture_menu_item = self.handshake_menuitem("Picture", "picture.png")
         menu = gtk.Menu()
         picture_menu_item.set_submenu(menu)
         self.popup_menu_workaround(menu)
@@ -999,7 +1027,7 @@ class GTKTrayMenuBase(object):
 
 
     def make_audiomenuitem(self):
-        audio_menu_item = self.menuitem("Audio", "audio.png")
+        audio_menu_item = self.handshake_menuitem("Audio", "audio.png")
         menu = gtk.Menu()
         audio_menu_item.set_submenu(menu)
         self.popup_menu_workaround(menu)
@@ -1354,13 +1382,14 @@ class GTKTrayMenuBase(object):
 
 
     def make_windowsmenuitem(self):
-        windows_menu_item = self.menuitem("Windows", "windows.png")
+        windows_menu_item = self.handshake_menuitem("Windows", "windows.png")
         menu = gtk.Menu()
         windows_menu_item.set_submenu(menu)
         self.popup_menu_workaround(menu)
         menu.append(self.make_raisewindowsmenuitem())
         menu.append(self.make_minimizewindowsmenuitem())
         menu.append(self.make_refreshmenuitem())
+        menu.append(self.make_reinitmenuitem())
         windows_menu_item.show_all()
         return windows_menu_item
 
@@ -1368,8 +1397,16 @@ class GTKTrayMenuBase(object):
         def force_refresh(*_args):
             log("force refresh")
             self.client.send_refresh_all()
+            self.client.reinit_window_icons()
         return self.handshake_menuitem("Refresh", "retry.png", None, force_refresh)
 
+    def make_reinitmenuitem(self):
+        def force_reinit(*_args):
+            log("force reinit")
+            self.client.reinit_windows()
+            self.client.reinit_window_icons()
+        return self.handshake_menuitem("Re-initialize", "reinitialize.png", None, force_reinit)
+
     def make_raisewindowsmenuitem(self):
         def raise_windows(*_args):
             for win in self.client._window_to_id.keys():
@@ -1387,7 +1424,7 @@ class GTKTrayMenuBase(object):
 
 
     def make_servermenuitem(self):
-        server_menu_item = self.menuitem("Server", "server.png")
+        server_menu_item = self.handshake_menuitem("Server", "server.png")
         menu = gtk.Menu()
         server_menu_item.set_submenu(menu)
         self.popup_menu_workaround(menu)
