@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -64,7 +64,7 @@ PYTHON2 = sys.version_info[0]==2
 PYTHON3 = sys.version_info[0]==3
 
 import struct
-BITS = struct.calcsize("P")*8
+BITS = struct.calcsize(b"P")*8
 
 
 if PYTHON2:
@@ -170,8 +170,11 @@ def platform_release(release):
         SYSTEMVERSION_PLIST = "/System/Library/CoreServices/SystemVersion.plist"
         try:
             import plistlib
-            with open(SYSTEMVERSION_PLIST, "rb") as f:
-                pl = plistlib.load(f)           #@UndefinedVariable
+            if PYTHON2:
+                pl = plistlib.readPlist('/System/Library/CoreServices/SystemVersion.plist')
+            else:
+                with open(SYSTEMVERSION_PLIST, "rb") as f:
+                    pl = plistlib.load(f)           #@UndefinedVariable
             return pl['ProductUserVisibleVersion']
         except Exception as e:
             get_util_logger().debug("platform_release(%s)", release, exc_info=True)
@@ -259,6 +262,20 @@ try:
 except Exception as e:
     pass
 
+
+def is_X11():
+    if PYTHON2:
+        return True
+    if OSX or WIN32:
+        return False
+    from xpra.x11.gtk3.gdk_bindings import is_X11_Display   #@UnresolvedImport
+    x11 = is_X11_Display()
+    return x11
+
+def is_Wayland():
+    return os.environ.get("WAYLAND_DISPLAY") or os.environ.get("XDG_SESSION_TYPE")=="wayland"
+
+
 def is_distribution_variant(variant=b"Debian", os_file="/etc/os-release"):
     if not POSIX:
         return False
@@ -421,6 +438,12 @@ def close_all_fds(exceptions=[]):
             # by listdir() is already closed.
             pass
 
+def use_tty():
+    from xpra.util import envbool
+    NOTTY = envbool("XPRA_NOTTY", False)
+    stdin = sys.stdin
+    return not os.environ.get("MSYSCON") and not NOTTY and stdin and stdin.isatty()
+
 
 def shellsub(s, subs={}):
     """ shell style string substitution using the dictionary given """
@@ -436,15 +459,11 @@ def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
             #replace "~/" with "~$actual_username/"
             return os.path.expanduser("~%s/%s" % (actual_username, s[2:]))
         return os.path.expanduser(s)
-    if actual_username:
-        HOME = os.path.expanduser("~%s" % actual_username)
-    else:
-        HOME = os.path.expanduser("~")
     from collections import OrderedDict
     d = OrderedDict(subs)
     d.update({
         "PID"   : os.getpid(),
-        "HOME"  : HOME,
+        "HOME"  : expanduser("~/"),
         })
     if os.name=="posix":
         d.update({
@@ -563,6 +582,39 @@ class DummyContextManager(object):
         return "DummyContextManager"
 
 
+#workaround incompatibility between paramiko and gssapi:
+class nomodule_context(object):
+
+    def __init__(self, module_name):
+        self.module_name = module_name
+        self.saved_module = sys.modules.get(module_name)
+        sys.modules[module_name] = None
+    def __enter__(self):
+        pass
+    def __exit__(self, *_args):
+        if sys.modules.get(self.module_name) == None:
+            if self.saved_module is None:
+                try:
+                    del sys.modules[self.module_name]
+                except KeyError:
+                    pass
+            else:
+                sys.modules[self.module_name] = self.saved_module
+    def __repr__(self):
+        return "nomodule_context(%s)" % self.module_name
+
+class umask_context(object):
+
+    def __init__(self, umask):
+        self.umask = umask        
+    def __enter__(self):
+        self.orig_umask = os.umask(self.umask)
+    def __exit__(self, *_args):
+        os.umask(self.orig_umask)
+    def __repr__(self):
+        return "umask_context(%s)" % self.umask
+
+
 def disable_stdout_buffering():
     import gc
     # Appending to gc.garbage is a way to stop an object from being
@@ -633,7 +685,11 @@ def pollwait(process, timeout=5):
 
 def which(command):
     from distutils.spawn import find_executable
-    return find_executable(command)
+    try:
+        return find_executable(command)
+    except Exception:
+        get_util_logger().debug("find_executable(%s)", command, exc_info=True)
+        return None
 
 def get_status_output(*args, **kwargs):
     import subprocess
@@ -709,8 +765,8 @@ def get_peercred(sock):
         log = get_util_logger()
         try:
             import socket
-            creds = sock.getsockopt(socket.SOL_SOCKET, SO_PEERCRED, struct.calcsize('3i'))
-            pid, uid, gid = struct.unpack('3i',creds)
+            creds = sock.getsockopt(socket.SOL_SOCKET, SO_PEERCRED, struct.calcsize(b'3i'))
+            pid, uid, gid = struct.unpack(b'3i',creds)
             log("peer: %s", (pid, uid, gid))
             return pid, uid, gid
         except Exception as  e:
