@@ -130,8 +130,8 @@ function XpraWindow(client, canvas_state, wid, x, y, w, h, metadata, override_re
 				'<span class="windowicon"><img id="windowicon' + String(wid) + '" /></span> '+
 				'<span class="windowtitle" id="title' + String(wid) + '">' + this.title + '</span> '+
 				'<span class="windowbuttons"> '+
-				'<span id="maximize' + String(wid) + '"><img src="../icons/maximize.png" /></span> '+
-				'<span id="close' + String(wid) + '"><img src="../icons/close.png" /></span> '+
+				'<span id="maximize' + String(wid) + '"><img src="icons/maximize.png" /></span> '+
+				'<span id="close' + String(wid) + '"><img src="icons/close.png" /></span> '+
 				'</span></div>');
 		// make draggable
 		jQuery(this.div).draggable({ cancel: "canvas" });
@@ -440,7 +440,7 @@ XpraWindow.prototype.set_metadata_safe = function(metadata) {
 			//remove any existing "wmclass-" classes not in the new wm_class list:
 			for (var i = 0; i < classes.length; i++) {
 				var tclass = ""+classes[i];
-				if (tclass.indexOf("wmclass-")===0 && wm_class && wm_class.indexOf(tclass)<0) {
+				if (tclass.indexOf("wmclass-")===0 && wm_class && !wm_class.includes(tclass)) {
 					jQuery(this.div).removeClass(tclass);
 				}
 			}
@@ -578,6 +578,11 @@ XpraWindow.prototype.toggle_maximized = function() {
  * Fullscreen / unfullscreen the window.
  */
 XpraWindow.prototype.set_fullscreen = function(fullscreen) {
+	//the browser itself:
+	//we can't bring attention to the fullscreen widget, ie:
+	//$("#fullscreen").fadeIn(100).fadeOut(100).fadeIn(100).fadeOut(100).fadeIn(100);
+	//because the window is about to cover the top bar...
+	//so just fullscreen the window:
 	if (this.fullscreen==fullscreen) {
 		return;
 	}
@@ -857,6 +862,12 @@ XpraWindow.prototype.set_cursor = function(encoding, w, h, xhot, yhot, img_data)
 };
 
 
+XpraWindow.prototype.eos = function() {
+	this._close_jsmpeg();
+	this._close_broadway();
+	this._close_video();
+}
+
 
 /**
  * This function draws the contents of the off-screen canvas to the visible
@@ -885,22 +896,27 @@ XpraWindow.prototype._init_broadway = function(enc_width, enc_height, width, hei
 	this.log("broadway decoder initialized");
 	this.broadway_paint_location = [0, 0];
 	this.broadway_decoder.onPictureDecoded = function(buffer, p_width, p_height, infos) {
-		me.debug("draw", "broadway picture decoded: ", buffer.length, "bytes, size ", p_width, "x", p_height+", paint location: ", me.broadway_paint_location,"with infos=", infos);
-		if(!me.broadway_decoder) {
-			return;
-		}
-		var img = me.offscreen_canvas_ctx.createImageData(p_width, p_height);
-		img.data.set(buffer);
-		var x = me.broadway_paint_location[0];
-		var y = me.broadway_paint_location[1];
-		me.offscreen_canvas_ctx.putImageData(img, x, y);
-		if(enc_width!=width || enc_height!=height) {
-			//scale it:
-			me.offscreen_canvas_ctx.drawImage(me.offscreen_canvas, x, y, p_width, p_height, x, y, width, height);
-		}
+		me._broadway_paint(buffer, enc_width, enc_height, width, height, p_width, p_height, infos);
 	};
 };
 
+XpraWindow.prototype._broadway_paint = function(buffer, enc_width, enc_height, width, height, p_width, p_height, infos) {
+	this.debug("draw", "broadway picture decoded: ", buffer.length, "bytes, size ", p_width, "x", p_height+", paint location: ", this.broadway_paint_location,"with infos=", infos);
+	if(!this.broadway_decoder) {
+		return;
+	}
+	var img = this.offscreen_canvas_ctx.createImageData(p_width, p_height);
+	img.data.set(buffer);
+	var x = this.broadway_paint_location[0];
+	var y = this.broadway_paint_location[1];
+	this.offscreen_canvas_ctx.putImageData(img, x, y);
+	if(enc_width!=width || enc_height!=height) {
+		//scale it:
+		this.offscreen_canvas_ctx.drawImage(this.offscreen_canvas, x, y, p_width, p_height, x, y, width, height);
+	}
+};
+
+
 XpraWindow.prototype._close_broadway = function() {
 	this.broadway_decoder = null;
 }
@@ -1068,8 +1084,31 @@ var DEFAULT_BOX_COLORS = {
         "vp9"     : "lavender",
         "mpeg4"   : "black",
         "scroll"  : "brown",
+        "mpeg1"   : "olive",
         }
 
+XpraWindow.prototype.get_jsmpeg_renderer = function get_jsmpeg_renderer() {
+	if (this.jsmpeg_renderer==null) {
+		var options = new Object();
+		//webgl is still buggy
+		//if (JSMpeg.Renderer.WebGL.IsSupported()) {
+		//	this.jsmpeg_renderer = new JSMpeg.Renderer.WebGL(options);
+		//}
+		//else {
+		this.jsmpeg_renderer = new JSMpeg.Renderer.Canvas2D(options);
+		//}
+	}
+	return this.jsmpeg_renderer;
+}
+
+XpraWindow.prototype._close_jsmpeg = function _close_jsmpeg() {
+	if (this.jsmpeg_renderer!=null) {
+		this.jsmpeg_renderer.destroy();
+	}
+	//decoder doesn't need cleanup?
+	this.jsmpeg_decoder = null;
+}
+
 XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_data, packet_sequence, rowstride, options, decode_callback) {
 	this.debug("draw", "do_paint(", img_data.length, " bytes of ", ("zlib" in options?"zlib ":""), coding, " data ", width, "x", height, " at ", x, ",", y, ") focused=", this.focused);
 	var me = this;
@@ -1088,6 +1127,7 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
 	}
 
 	function painted(skip_box) {
+		img_data = null;
 		me.paint_pending = 0;
 		if (me.paint_debug && !skip_box) {
 			var color = DEFAULT_BOX_COLORS[coding] || "white";
@@ -1097,6 +1137,7 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
 	}
 
 	function paint_error(e) {
+		img_data = null;
 		me.error("error painting", coding, e);
 		me.paint_pending = 0;
 		decode_callback(""+e);
@@ -1161,6 +1202,36 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
 			}
 			j.src = "data:image/"+coding+";base64," + Utilities.ArrayBufferToBase64(img_data);
 		}
+		else if (coding=="mpeg1") {
+			var frame = options["frame"] || 0;
+			if (frame==0 || this.jsmpeg_decoder==null) {
+				var options = new Object();
+				options.streaming = true;
+				options.decodeFirstFrame = false;
+				this.jsmpeg_decoder = new JSMpeg.Decoder.MPEG1Video(options);
+				//TODO: instead of delegating, we should probably subclass the renderer
+				// (but which one! GL or not?):
+				var renderer = new Object();
+				renderer.render = function render(Y, Cr, Cb) {
+					var jsmpeg_renderer = me.get_jsmpeg_renderer();
+					jsmpeg_renderer.render(Y, Cr, Cb);
+					var canvas = jsmpeg_renderer.canvas;
+					me.offscreen_canvas_ctx.drawImage(canvas, x, y, width, height);
+					paint_box("olive", x, y, width, height);
+				}
+				renderer.resize = function resize(newWidth, newHeight) {
+					var jsmpeg_renderer = me.get_jsmpeg_renderer();
+					jsmpeg_renderer.resize(newWidth, newHeight);
+				}
+				this.jsmpeg_decoder.connect(renderer);
+			}
+			var pts = frame;
+			this.jsmpeg_decoder.write(pts, img_data);
+			var decoded = this.jsmpeg_decoder.decode();
+			this.debug("draw", coding, "frame", frame, "data len=", img_data.length, "decoded=", decoded);
+			//TODO: only call painted when we have actually painted the frame?
+			painted();
+		}
 		else if (coding=="h264") {
 			var frame = options["frame"] || 0;
 			if(frame==0) {
@@ -1244,6 +1315,7 @@ XpraWindow.prototype.do_paint = function paint(x, y, width, height, coding, img_
  */
 XpraWindow.prototype.destroy = function destroy() {
 	// remove div
+	this._close_jsmpeg();
 	this._close_broadway();
 	this._close_video();
 	this.div.remove();
